from office365.runtime.auth.authentication_context import AuthenticationContext
from office365.sharepoint.client_context import ClientContext
from office365.sharepoint.files.file import File
import os
import csv
from io import BytesIO
import pandas as pd

class da_tran_SP365:
    def __init__(self, site_url, client_id, client_secret):
        """Create connection to Sharepoint Site"""
        self.site_url = site_url
        self.client_id = client_id
        self.client_secret = client_secret
        ctx_auth = AuthenticationContext( self.site_url)
        ctx_auth.acquire_token_for_app( self.client_id , self.client_secret )
        self.ctx = ClientContext(self.site_url, ctx_auth)
        web = self.ctx.web
        self.ctx.load(web)
        self.ctx.execute_query()
        print('Authen to {} OK'.format(web.properties['Title']))

    def download(self, sharepoint_location, local_location = '', as_dataframe = False, sheet_name = None) :
        """Download file from sharepoint or Read Excel/csv from sharepoint as pd dataframe"""
        response = File.open_binary(self.ctx, sharepoint_location ) # save file from sharepoint as binary
        if str(response.status_code) == '200' :
            if as_dataframe :
                toread = BytesIO()
                toread.write(response.content)  # pass your `decrypted` string as the argument here
                toread.seek(0)  # reset the pointer
                if '.csv' in sharepoint_location : return pd.read_csv(toread)
                else : return pd.read_excel(toread, sheet_name = sheet_name)
            else :
                with open(local_location, "wb") as local_file: 
                    local_file.write(response.content) # write in your pc
                print('Download OK File')
        else :
            raise Exception('Cannot Download File')

    def upload(self, sharepoint_location, local_location):
        """Upload file to sharepoint"""
        with open(local_location, 'rb') as content_file: 
            file_content = content_file.read() # read file from your pc
        dir_, name = os.path.split(sharepoint_location)
        self.ctx.web.get_folder_by_server_relative_url(dir_).upload_file(name, file_content).execute_query() # upload file to sharepoint

    def read_list(self, list_title, local_location = '', as_dataframe = False):
        """Read list from Sharepoint and Download as csv or pandas dataframe"""
        list_to_export = self.ctx.web.lists.get_by_title(list_title)
        list_items = list_to_export.items.get().execute_query()
        self.list_items = list_items
        if len(list_items) == 0: print("No data found")
        else :
            if as_dataframe :
                df_in = pd.DataFrame()
                for i in list_items : df_in = df_in.append(pd.DataFrame(data = [list(i.properties.values())], columns = i.properties.keys()))
                return df_in
            elif local_location == '' : raise Exception("Please input your savefile's name")
            else :
                with open(local_location, 'w',newline='') as fh:
                    fields = list_items[0].properties.keys()
                    w = csv.DictWriter(fh, fields)
                    w.writeheader()
                    for item in list_items: w.writerow(item.properties)
                print('Download List OK')


import pandas as pd
from datetime import datetime
from sqlalchemy import create_engine

def dump_replace_existsame(df_in, con_dict, table_name_in,filter_in) :
    """Create connection to database, delete exists row of table in database with key filter_in and dump df_in append to table"""
    print(table_name_in,' Start ',datetime.now())
    #Create connection to database
    connection_str = str(con_dict['type'] + '+' + con_dict['driver'] 
                    + '://' + con_dict['user'] + ':' + con_dict['pass'] + '@' + con_dict['host'] 
                    + ':' + con_dict['port'] + '/' + con_dict['database'])

    engine = create_engine(connection_str)
    sql_q = 'delete from ' + table_name_in + ' where '

    #Create SQL Query for Delete
    i = 0
    while i < len(filter_in) :
        filter_name = filter_in[i]
        filter_filter = tuple(df_in[filter_name].astype('str').unique())
        #prevent error for 1 filter row by add ()
        if len(filter_filter) == 1 :
            filter_filter = "('" + filter_filter[0] + "')"
        else :
            filter_filter = str(filter_filter)

        #Add "and" for second columns 
        if i > 0 :
            sql_q = sql_q + ' and '

        sql_q = sql_q + '[' + filter_name  + ']' + ' in ' + filter_filter
        i += 1

    #Delete exiting row from table
    try :
        print('start delete')
        engine.execute(sql_q)
        print('Delete Last '+str(filter_in)+' at',datetime.now())
    except :
        print('Delete error or Do not have table to delete at',datetime.now())
        pass

    #Dump df_in append to database
    df_in.to_sql(table_name_in,con = engine,index = False,if_exists = 'append',chunksize = 150, method = 'multi')
    print(table_name_in,' End ',datetime.now())
    return True

def dump_only_new(df_in, con_dict, table_name_in,filter_in) :
    """Create connection to database, delete exists row of df that has key filter_in and dump df_in append to table --not test yet"""
    print(table_name_in,' Start ',datetime.now())
    #Create connection to database
    connection_str = str(con_dict['type'] + '+' + con_dict['driver'] 
                    + '://' + con_dict['user'] + ':' + con_dict['pass'] + '@' + con_dict['host'] 
                    + ':' + con_dict['port'] + '/' + con_dict['database'])

    engine = create_engine(connection_str)

    #Loop delete --may get error , not test yet
    for i_in in filter_in :
        #get key values from database
        filter_name = filter_in
        sql_q = 'select distinct '+ '[' +  filter_name + ']' +' from ' + table_name_in
        filter_filter = pd.read_sql_query(sql_q, con = engine111)

        #delete existing key from df_in
        filter_filter = ~df_in[filter_name].astype('str').isin(filter_filter[filter_name].astype('str'))
        df_in = df_in[filter_filter]

    #Dump df_in append to database
    df_in.to_sql(table_name_in,con = engine111,index = False,if_exists = 'append',chunksize = 150, method = 'multi')
    print(table_name_in,' End ',datetime.now())
    return True

# # Sample 
"""
config_db = {
            'type' :'BAHBAHBAH',  # type of database you use
            'driver':'BAHBAHBAH',  # type of driver you use
            'user' : 'BAHBAHBAH', # user to connect database
            'pass' : 'BAHBAHBAH', # password to connect database
            'host' :'BAHBAHBAH' , # server's name or server's ip
            'port' : 'BAHBAHBAH', # port number
            'database' : 'BAHBAHBAH' # database's name
            }

df = pd.DataFrame({'BAH':[1,2,3,4],'BAHBAH':[1,2,3,4],'BAHBAHBAH':[1,2,3,4]}) # Pandas dataframe
filter_in = ['BAH' , 'BAHBAH' , 'BAHBAHBAH'] # List of Key column
table = 'BAHBAHBAH' # Table's name

dump_replace_existsame(df, config_db, table ,filter_in)
dump_only_new(df, config_db, table ,filter_in)
"""



import email, smtplib, ssl
from email import encoders
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.image import MIMEImage
from datetime import datetime
import os
import csv
import re

class email_sender :
    def __init__(self, user, password, servername):
        """Store username , password and servername"""
        self.servername = servername
        self.password = password
        self.sender = user
        
    def executor(self,recipients, message ):    
        """Connect to Server and Send Email, will return status of sending in text""" 
        #Create Connection
        smtpObj = smtplib.SMTP(self.servername, timeout = 30)    
        smtpObj.login(self.sender.split('@')[0], self.password)
        ssl.create_default_context()
        try :
            #Send and Quit
            smtpObj.sendmail(self.sender, recipients, message.as_string())
            smtpObj.quit()
            return 'Send OK'
        except Exception as e:
            #Quit if error
            smtpObj.quit()
            return str(e)
        
    def send_email(self, sendto,subject,text_in):
        """Create Message Part"""
        #split for multple
        recipients = sendto.split(";")

        #Create Head of Email
        message = MIMEMultipart()
        message["From"] = self.sender
        message["To"] =  ', '.join(recipients)
        message["Subject"] = subject

        #Create Message Part
        html = """<html><body><p>{}</p></body></html>""".format(text_in)

        #Attach before send
        partHTML = MIMEText(html, 'html')
        message.attach(partHTML)

        #Sending Email and return status text
        return self.executor(recipients, message)

def log_csv(file_name,msg_in):
    logic = True
    if os.path.exists(file_name) : logic = False
    if '/' in file_name : 
        if file_name.split('/')[0] != '' :
            print(file_name.split('/')[0])
            dir_name = re.sub('(/.*\..*)','',file_name)
            if not os.path.isdir(dir_name) : os.mkdir(dir_name)
    with open(file_name.replace('/',''), 'a', newline='') as csvfile:
        fieldnames = ['Timestamp','msg']
        writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
        if logic : writer.writeheader()
        writer.writerow({'Timestamp':str(datetime.now()),'msg':msg_in})


from office365.runtime.auth.authentication_context import AuthenticationContext
from office365.sharepoint.client_context import ClientContext
from office365.sharepoint.files.file import File
import os
import csv
from io import BytesIO
import pandas as pd

class da_tran_SP365:
    def __init__(self, site_url, client_id, client_secret):
        """Create connection to Sharepoint Site"""
        self.site_url = site_url
        self.client_id = client_id
        self.client_secret = client_secret
        ctx_auth = AuthenticationContext( self.site_url)
        ctx_auth.acquire_token_for_app( self.client_id , self.client_secret )
        self.ctx = ClientContext(self.site_url, ctx_auth)
        web = self.ctx.web
        self.ctx.load(web)
        self.ctx.execute_query()
        print('Authen to {} OK'.format(web.properties['Title']))

    def download(self, sharepoint_location, local_location = '', as_dataframe = False, sheet_name = None) :
        """Download file from sharepoint or Read Excel/csv from sharepoint as pd dataframe"""
        response = File.open_binary(self.ctx, sharepoint_location ) # save file from sharepoint as binary
        if str(response.status_code) == '200' :
            if as_dataframe :
                toread = BytesIO()
                toread.write(response.content)  # pass your `decrypted` string as the argument here
                toread.seek(0)  # reset the pointer
                if '.csv' in sharepoint_location : return pd.read_csv(toread)
                else : return pd.read_excel(toread, sheet_name = sheet_name)
            else :
                with open(local_location, "wb") as local_file: 
                    local_file.write(response.content) # write in your pc
                print('Download OK File')
        else :
            raise Exception('Cannot Download File')

    def upload(self, sharepoint_location, local_location):
        """Upload file to sharepoint"""
        with open(local_location, 'rb') as content_file: 
            file_content = content_file.read() # read file from your pc
        dir_, name = os.path.split(sharepoint_location)
        self.ctx.web.get_folder_by_server_relative_url(dir_).upload_file(name, file_content).execute_query() # upload file to sharepoint

    def read_list(self, list_title, local_location = '', as_dataframe = False):
        """Read list from Sharepoint and Download as csv"""
        list_to_export = self.ctx.web.lists.get_by_title(list_title)
        list_items = list_to_export.items.get().execute_query()
        self.list_items = list_items
        if len(list_items) == 0:
            print("No data found")
        else :
            if as_dataframe :
                pass
                #return
            else :
                with open(local_location, 'w',newline='') as fh:
                    fields = list_items[0].properties.keys()
                    w = csv.DictWriter(fh, fields)
                    w.writeheader()
                    for item in list_items:
                        w.writerow(item.properties)
                print('Download List OK')


# # Download File
def Download_SP365(dict_in) :
    """Download file from sharepoint as described in dict_in"""
    file_part_sp = dict_in['folder_sp'] + dict_in['file_name_sp'] # create path to sharepoint
    file_part_local = dict_in['folder_local'] + dict_in['file_name_local'] # create path to your pc
    url = dict_in['site_url'] # get site url
    ctx_auth = AuthenticationContext(url)
    ctx_auth.acquire_token_for_app(dict_in['cli_id']
                                   , dict_in['cli_secret'])
    ctx = ClientContext(url, ctx_auth)
    response = File.open_binary(ctx, file_part_sp ) # save file from sharepoint as binary
    with open(file_part_local, "wb") as local_file: 
        local_file.write(response.content) # write in your pc
    return True

# # Sample of Download_SP365
"""
import_sp_in = {
                'file_name_sp' : 'BAHBAHBAH' , # your file name on sharepoint
                'file_name_local' : 'BAHBAHBAH' , # the file name you want to keep on your pc
                'folder_sp' : '/sites/BAHBAHBAH' , # the subfolder name on sharepoint
                'folder_local' : 'BAHBAHBAH' , # the folder name you want to keep file on your pc
                'site_url' : 'BAHBAHBAH/sites/BAHBAHBAH' , # site's URL
                'cli_id' : 'BAHBAHBAH' , # Client ID Generated from Sharepoint
                'cli_secret' : 'BAHBAHBAH' # Client Secret Generated from Sharepoint
                }

Download_SP365(import_sp_in)
"""

# # Upload File
def Upload_SP365(dict_in):
    """Upload file to sharepoint as described in dict_in"""
    url = dict_in['site_url'] # get site url
    localpath = dict_in['folder_local'] + dict_in['file_name_local']  # create path to your pc
    remotepath = dict_in['folder_sp'] + dict_in['file_name_sp']  # create path to sharepoint
    ctx_auth = AuthenticationContext(url)
    ctx_auth.acquire_token_for_app(dict_in['cli_id']
                                       , dict_in['cli_secret'])
    ctx = ClientContext(url, ctx_auth) 
    with open(localpath, 'rb') as content_file: 
        file_content = content_file.read() # read file from your pc
    dir_, name = os.path.split(remotepath)
    file = ctx.web.get_folder_by_server_relative_url(dir_).upload_file(name, file_content).execute_query() # upload file to sharepoint
    return True

# # Sample of Upload_SP365
"""
export_sp_out = {
                'file_name_sp' : 'BAHBAHBAH' , # your file name on sharepoint
                'file_name_local' : 'BAHBAHBAH' , # the file name you want to keep on your pc
                'folder_sp' : '/sites/BAHBAHBAH' , # the subfolder name on sharepoint
                'folder_local' : 'BAHBAHBAH' , # the folder name you want to keep file on your pc
                'site_url' : 'BAHBAHBAH/sites/BAHBAHBAH' , # site's URL
                'cli_id' : 'BAHBAHBAH' , # Client ID Generated from Sharepoint
                'cli_secret' : 'BAHBAHBAH' # Client Secret Generated from Sharepoint
                }

Upload_SP365(export_sp_out)
"""


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py_topping",
    version="0.1.1",
    license = "MIT",	
    author="Chanon Krittapholchai",
    author_email="chanon.krittapholchai@gmail.com",
    description="simplify functions from other libraries function in 1-2 lines",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/chanon-kr/Shared_Function",
    download_url = "https://github.com/chanon-kr/Shared_Function/archive/0.1.1.tar.gz",
    keywords = ["utility"],
    packages=setuptools.find_packages(),
    classifiers=[
        "Development Status :: 1 - Planning",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
    python_requires = '>=3.7'
)


import pandas as pd
from sqlalchemy import create_engine

class da_tran_SQL :
    """interact with SQL"""
    def __init__(self, sql_type, host_name, database_name, user, password , **kwargs):
        """Create connection to SQL Server"""

        type_dic = {'MSSQL' : ['mssql', 'pymssql', '1433']}
        
        if type_dic.get(sql_type,'Error') == 'Error' :
            raise Exception("Please Insert Type of your Database with these range \n{}".format(list(type_dic.keys())))
        
        connection_str = str(type_dic[sql_type][0] + '+' + kwargs.get('driver',type_dic[sql_type][1])
                           + '://' + user + ':' + password + '@' + host_name
                           + ':' + kwargs.get('port',type_dic[sql_type][2]) + '/' + database_name)
        self.engine = create_engine(connection_str)
        print(pd.read_sql_query("""SELECT 'Connection OK'""", con = self.engine).iloc[0,0]) 

    def read(self, table_name_in, condition_in = '', SP = False, param = ''):
        """Read Table or View or Store Procedure"""
        if SP :
            sql_q = """EXEC """ + table_name_in + ' '
            if param == '' : raise Exception("Please insert SP's parameter")
            elif type(param) != dict : raise Exception("param must be dict")
            else :
                n = 0
                for i in param.keys() :
                    if type(param[i]) == str : param[i] = """'{}'""".format(param[i])
                    if n > 0 : sql_q += ' , '
                    sql_q += """ {} = {}""".format(i, param[i])
                    n += 1
        else :
            sql_q = """SELECT * FROM [{}]""".format(table_name_in)
            if not condition_in == '' :
                sql_q += """ WHERE """ + condition_in
        return pd.read_sql_query(sql_q , con = self.engine)

    def write_logic_sql(self, key , value_in):
        """Write SQL Condition Query that include >, <, ="""
        logic_query = '['+ str(key) + '] ' + value_in['logic'] + ' '
        value_in['type'] = value_in.get('type','')
        if value_in.get('value','there is no value') != 'there is no value' :
            if ('date' in value_in['type']) | ('time' in value_in['type']):  # Datetime will need '' in SQL Statement
                logic_query += """'""" + str(value_in['value']) + """'"""
            else : logic_query += str(value_in['value'])
        else : raise Exception("Please insert value data")  # will get error if not specific value
        return logic_query

    def dump_whole(self, df_in, table_name_in) :
        """Delete exists table and replace with new df"""
        print('Start Filter Existing data from df at ',pd.Timestamp.now())
        #Dump df_in to database
        df_in.to_sql(table_name_in,con = self.engine,index = False,if_exists = 'replace',chunksize = 150, method = 'multi')
        print('Dump data to ',table_name_in,' End ',pd.Timestamp.now())

    def write_in_sql(self,df_in , key) :
        """Write SQL Condition Query 'in (x,x,x)'"""
        filter_filter = tuple(df_in[key].astype('str').unique())
        if len(filter_filter) == 1 : 
            filter_filter = str(filter_filter).replace(',)',')')  # tuple with 1 value will be ( x , ) => need to convert
            logic_query = '[' + key  + ']' + ' in ' + filter_filter
        elif len(filter_filter) > 1 :
            filter_filter = str(filter_filter) # tuple with > 1 values will be ( x, y, z) which can be use in SQL
            logic_query = '[' + key  + ']' + ' in ' + filter_filter
        else : logic_query = '' # Return Nothing
        return logic_query

    def dump_replace(self, df_in, table_name_in, list_key, math_logic = '',debug = False):
        """Delete exists row of table in database with same key(s) as df and dump df append to table"""
        #Create SQL Query for Delete
        sql_q = 'delete from ' + table_name_in + ' where '

        if math_logic != '' : logic_list = list(math_logic.keys())
        else : logic_list = ['There_is_no_logic_math_need_to_write'] # column in logic_list will never be true

        #for single key
        if 'str' in str(type(list_key))  :
            if not list_key in df_in.columns :
                raise Exception("{} is not in df's columns".format(list_key))
            if list_key in logic_list :
                sql_q += self.write_logic_sql(list_key, math_logic[list_key])
            else :  
                sql_q += self.write_in_sql(df_in,list_key)

        #for multi keys    
        elif 'list' in str(type(list_key)) :
            if not min(x in df_in.columns for x in list_key) :
                raise Exception("Some keys are not in df's columns")
            i = 0
            while i < len(list_key) :
                filter_name = list_key[i]
                if filter_name in logic_list :
                    sql_q += self.write_logic_sql(filter_name, math_logic[filter_name])
                else :
                    sql_q += self.write_in_sql(df_in,filter_name)
                i += 1
                if i < len(list_key) : sql_q += ' and '

        #if key is not either string or list
        else :
            raise Exception("List of Key must be string or list")
        
        #Delete exiting row from table
        try :
            print('Start delete old data at',pd.Timestamp.now())
            self.engine.execute(sql_q)
            print('Delete Last '+str(list_key)+' at',pd.Timestamp.now())
        except Exception as e:
            print('Delete error or Do not have table to delete at',pd.Timestamp.now())
            if debug : print(e)
            pass
            
        if debug : print(sql_q)
        #Dump df_in append to database
        df_in.to_sql(table_name_in,con = self.engine,index = False,if_exists = 'append',chunksize = 150, method = 'multi')
        print('Dump data to ',table_name_in,' End ',pd.Timestamp.now())

    def dump_new(self, df_in, table_name_in, list_key , debug = False) :
        """Delete exists row of df that has same key(s) as table and dump df_in append to table"""
        print('Start Filter Existing data from df at ',pd.Timestamp.now())

        #for single key
        if 'str' in str(type(list_key))  :
            if not list_key in df_in.columns :
                raise Exception("{} is not in df's columns".format(list_key))       
            sql_q = 'select distinct '+ '[' +  list_key + ']' +' from ' + table_name_in
            filter_filter = pd.read_sql_query(sql_q, con = self.engine).iloc[:,0]
            logic_filter = ~df_in[list_key].isin(filter_filter)

        #for multi keys    
        elif 'list' in str(type(list_key)) :
            if not min(x in df_in.columns for x in list_key) :
                raise Exception("Some keys are not in df's columns")
            i = 0
            df_in['key_sql_filter'] = ''
            sql_q = 'SELECT DISTINCT '
            while i < len(list_key) :
                filter_name = list_key[i]
                if i != 0 : sql_q += ' , '
                df_in['key_sql_filter'] = df_in['key_sql_filter'].astype('str') + df_in[filter_name].astype('str')
                sql_q += '[' +  filter_name + '] ' 
                i += 1
            sql_q += ' FROM ' + table_name_in
            if debug : print(sql_q)
            filter_filter = pd.read_sql_query(sql_q, con = self.engine)
            filter_filter['key_sql_filter'] = ''
            for i in filter_filter.columns :
                if  i == 'key_sql_filter' : pass
                else : filter_filter['key_sql_filter'] = filter_filter['key_sql_filter'].astype('str') + filter_filter[i].astype('str')
            if debug : print(filter_filter.head())
            filter_filter = filter_filter['key_sql_filter'].unique()
            if debug : print('======filter======\n',filter_filter,'\n======in======\n',df_in.head())
            logic_filter = ~df_in['key_sql_filter'].isin(filter_filter)
            df_in = df_in.drop('key_sql_filter', axis = 1)

        #if key is not either string or list
        else :
            raise Exception("List of Key must be string or list")

        df_in = df_in[logic_filter]

        #Dump df_in append to database
        df_in.to_sql(table_name_in,con = self.engine,index = False,if_exists = 'append',chunksize = 150, method = 'multi')
        print('Dump data to ',table_name_in,' End ', pd.Timestamp.now())

    def dump_new_old(self, df_in, table_name_in, list_key) :
        """Delete exists row of df that has same key(s) as table and dump df_in append to table"""
        print('Start Filter Existing data from df at ',pd.Timestamp.now())

        #for single key
        if 'str' in str(type(list_key))  :
            if not list_key in df_in.columns :
                raise Exception("{} is not in df's columns".format(list_key))       
            sql_q = 'select distinct '+ '[' +  list_key + ']' +' from ' + table_name_in
            filter_filter = pd.read_sql_query(sql_q, con = self.engine).iloc[:,0]
            logic_filter = ~df_in[list_key].isin(filter_filter)

        #for multi keys    
        elif 'list' in str(type(list_key)) :
            if not min(x in df_in.columns for x in list_key) :
                raise Exception("Some keys are not in df's columns")
            i = 0
            while i < len(list_key) :
                filter_name = list_key[i]
                sql_q = 'select distinct '+ '[' +  filter_name + ']' +' from ' + table_name_in
                filter_filter = pd.read_sql_query(sql_q, con = self.engine).iloc[:,0]
                if i == 0 : logic_filter = df_in[filter_name].isin(filter_filter)
                else : logic_filter = (logic_filter) & (df_in[filter_name].isin(filter_filter))
                i += 1

            logic_filter = ~logic_filter

        #if key is not either string or list
        else :
            raise Exception("List of Key must be string or list")

        df_in = df_in[logic_filter]

        #Dump df_in append to database
        df_in.to_sql(table_name_in,con = self.engine,index = False,if_exists = 'append',chunksize = 150, method = 'multi')
        print('Dump data to ',table_name_in,' End ', pd.Timestamp.now())